@@ -2,7 +2,8 @@
 
 from curses import ascii
 
-from fontbro.features import FEATURES as _FEATURES_DATA
+from fontbro.features import FEATURES as _FEATURES_LIST
+from fontbro.flags import get_flag, set_flag
 
 from fontTools import unicodedata
 from fontTools.subset import parse_unicodes, Subsetter
@@ -19,14 +20,14 @@ import sys
 
 class Font(object):
     """
-    human-friendly font operations.
+    friendly font operations.
     """
 
     # Features:
     # https://docs.microsoft.com/en-gb/typography/opentype/spec/featurelist
     # https://developer.mozilla.org/en-US/docs/Web/CSS/font-feature-settings
-    _FEATURES = _FEATURES_DATA
-    _FEATURES_DICT = {feature['tag']: feature for feature in _FEATURES}
+    _FEATURES_LIST = _FEATURES_LIST
+    _FEATURES_BY_TAG = {feature['tag']: feature for feature in _FEATURES_LIST}
 
     # Formats:
     FORMAT_OTF = 'otf'
@@ -96,18 +97,25 @@ class Font(object):
     _NAMES_BY_KEY = {item['key']: item for item in _NAMES}
 
     # Style Flags:
+    # https://docs.microsoft.com/en-us/typography/opentype/spec/head
+    # https://docs.microsoft.com/en-us/typography/opentype/spec/os2#fsselection
+    STYLE_FLAG_REGULAR = 'regular'
     STYLE_FLAG_BOLD = 'bold'
     STYLE_FLAG_ITALIC = 'italic'
-    STYLE_FLAG_OUTLINE = 'outline'
-    STYLE_FLAG_REGULAR = 'regular'
     STYLE_FLAG_UNDERLINE = 'underline'
-
+    STYLE_FLAG_OUTLINE = 'outline'
+    STYLE_FLAG_SHADOW = 'shadow'
+    STYLE_FLAG_CONDENSED = 'condensed'
+    STYLE_FLAG_EXTENDED = 'extended'
     _STYLE_FLAGS = {
+        STYLE_FLAG_REGULAR: {'bit_head_mac': None, 'bit_os2_fs': 6},
         STYLE_FLAG_BOLD: {'bit_head_mac': 0, 'bit_os2_fs': 5},
         STYLE_FLAG_ITALIC: {'bit_head_mac': 1, 'bit_os2_fs': 0},
+        STYLE_FLAG_UNDERLINE: {'bit_head_mac': 2, 'bit_os2_fs': None},
         STYLE_FLAG_OUTLINE: {'bit_head_mac': 3, 'bit_os2_fs': 3},
-        STYLE_FLAG_REGULAR: {'bit_head_mac': None, 'bit_os2_fs': 6},
-        STYLE_FLAG_UNDERLINE: {'bit_head_mac': 2, 'bit_os2_fs': 1},
+        STYLE_FLAG_SHADOW: {'bit_head_mac': 4, 'bit_os2_fs': None},
+        STYLE_FLAG_CONDENSED: {'bit_head_mac': 5, 'bit_os2_fs': None},
+        STYLE_FLAG_EXTENDED: {'bit_head_mac': 6, 'bit_os2_fs': None},
     }
     _STYLE_FLAGS_KEYS = _STYLE_FLAGS.keys()
 
@@ -128,7 +136,7 @@ class Font(object):
         {'tag': 'SOFT', 'name': 'Softness'},
         {'tag': 'WONK', 'name': 'Wonky'},
     ]
-    _VARIABLE_AXES_DICT = {axis['tag']: axis for axis in _VARIABLE_AXES}
+    _VARIABLE_AXES_BY_TAG = {axis['tag']: axis for axis in _VARIABLE_AXES}
 
     # Weights:
     # https://docs.microsoft.com/en-us/typography/opentype/otspec170/os2#usweightclass
@@ -144,7 +152,7 @@ class Font(object):
     WEIGHT_EXTRA_BOLD = 'Extra-bold'  # (Ultra-bold)
     WEIGHT_BLACK = 'Black'  # (Heavy)
     WEIGHT_EXTRA_BLACK = 'Extra-black'  # (Nord)
-    _WEIGHT_DICT = {
+    _WEIGHTS_BY_VALUE = {
         50: WEIGHT_EXTRA_THIN,
         100: WEIGHT_THIN,
         200: WEIGHT_EXTRA_LIGHT,
@@ -170,7 +178,7 @@ class Font(object):
     WIDTH_EXPANDED = 'Expanded'
     WIDTH_EXTRA_EXPANDED = 'Extra-expanded'
     WIDTH_ULTRA_EXPANDED = 'Ultra-expanded'
-    _WIDTH_DICT = {
+    _WIDTHS_BY_VALUE = {
         1: WIDTH_ULTRA_CONDENSED,
         2: WIDTH_EXTRA_CONDENSED,
         3: WIDTH_CONDENSED,
@@ -249,19 +257,29 @@ class Font(object):
         """
         Gets the font opentype features.
 
-        :returns: The features.
+        :returns: The features list.
         :rtype: list of dict
         """
+        features_tags = self.get_features_tags()
+        return [
+            self._FEATURES_BY_TAG.get(features_tag).copy()
+            for features_tag in features_tags
+            if features_tag in self._FEATURES_BY_TAG
+        ]
+
+    def get_features_tags(self):
+        """
+        Gets the font opentype features tags.
+
+        :returns: The features tags list.
+        :rtype: list of str
+        """
         font = self.get_ttfont()
         gsub = font.get('GSUB')
         if gsub:
             feature_record = gsub.table.FeatureList.FeatureRecord or []
             features_tags = [feature.FeatureTag for feature in feature_record]
-            return [
-                self._FEATURES_DICT.get(features_tag).copy()
-                for features_tag in features_tags
-                if features_tag in self._FEATURES_DICT
-            ]
+            return features_tags
         return []
 
     def get_format(self, ignore_flavor=False):
@@ -312,12 +330,11 @@ class Font(object):
 
         :raises KeyError: if the key is not a valid name key/id
         """
-        name_id = self._get_name_id(key)
         font = self.get_ttfont()
-        for record in font['name'].names:
-            if record.nameID == name_id:
-                return f'{record}'
-        raise KeyError(f'Invalid name key: {key}.')
+        name_id = self._get_name_id(key)
+        name_table = font['name']
+        name = name_table.getName(name_id, 3, 1)
+        return name.toUnicode() if name else None
 
     def get_names(self):
         """
@@ -390,24 +407,32 @@ class Font(object):
         ]
         return scripts
 
-    @classmethod
-    def _get_bool_flag(self, bits, bit):
-        return bool(bits & (1 << bit))
+    def get_style_flag(self, key):
+        """
+        Gets the style flag reading OS/2 and macStyle tables.
 
-    def _get_style_flag_by_bits(self, bit_os2_fs, bit_head_mac):
+        :param key: The key
+        :type key: string
+
+        :returns: The style flag.
+        :rtype: bool
+        """
         font = self.get_ttfont()
+        bits = self._STYLE_FLAGS[key]
+        bit_os2_fs = bits['bit_os2_fs']
+        bit_head_mac = bits['bit_head_mac']
         # https://docs.microsoft.com/en-us/typography/opentype/spec/os2#fsselection
         flag_os2_fs = False
         if bit_os2_fs is not None:
             os2 = font.get('OS/2')
             if os2:
-                flag_os2_fs = self._get_bool_flag(os2.fsSelection, bit_os2_fs)
+                flag_os2_fs = get_flag(os2.fsSelection, bit_os2_fs)
         # https://developer.apple.com/fonts/TrueType-Reference-Manual/RM06/Chap6head.html
         flag_head_mac = False
         if bit_head_mac is not None:
             head = font.get('head')
             if head:
-                flag_head_mac = self._get_bool_flag(head.macStyle, bit_head_mac)
+                flag_head_mac = get_flag(head.macStyle, bit_head_mac)
         return flag_os2_fs or flag_head_mac
 
     def get_style_flags(self):
@@ -417,10 +442,7 @@ class Font(object):
         :returns: The dict representing the style flags.
         :rtype: dict
         """
-        return {
-            key: self._get_style_flag_by_bits(**self._STYLE_FLAGS.get(key))
-            for key in self._STYLE_FLAGS_KEYS
-        }
+        return {key: self.get_style_flag(key) for key in self._STYLE_FLAGS_KEYS}
 
     def get_ttfont(self):
         """
@@ -444,7 +466,7 @@ class Font(object):
         return [
             {
                 'tag': axis.axisTag,
-                'name': self._VARIABLE_AXES_DICT.get(axis.axisTag, {}).get(
+                'name': self._VARIABLE_AXES_BY_TAG.get(axis.axisTag, {}).get(
                     'name', axis.axisTag.title()
                 ),
                 'minValue': axis.minValue,
@@ -553,9 +575,9 @@ class Font(object):
             return None
         weight = os2.usWeightClass
         weight = min(max(0, weight), 1000)
-        weights = sorted(self._WEIGHT_DICT.keys())
+        weights = sorted(self._WEIGHTS_BY_VALUE.keys())
         closest_weight = min(weights, key=lambda weight_item: abs(weight_item - weight))
-        weight_name = self._WEIGHT_DICT.get(closest_weight)
+        weight_name = self._WEIGHTS_BY_VALUE.get(closest_weight)
         return {'name': weight_name, 'value': weight}
 
     def get_width(self):
@@ -571,7 +593,7 @@ class Font(object):
             return None
         width = os2.usWidthClass
         width = min(max(0, width), 9)
-        width_name = self._WIDTH_DICT.get(width)
+        width_name = self._WIDTHS_BY_VALUE.get(width)
         return {'name': width_name, 'value': width}
 
     def is_static(self):
@@ -593,6 +615,41 @@ class Font(object):
         font = self.get_ttfont()
         return 'fvar' in font
 
+    def rename(self, family_name='', style_name=''):
+        """
+        Renames the font names records according to the given family_name and style_name (subfamily_name).
+        If family_name is not defined it will be auto-detected.
+        If style_name is not defined it will be auto-detected.
+
+        :param family_name: The family name
+        :type family_name: str
+        :param style_name: The style name
+        :type style_name: str
+        """
+        family_name = (
+            family_name
+            or self.get_name(self.NAME_TYPOGRAPHIC_FAMILY_NAME)
+            or self.get_name(self.NAME_FAMILY_NAME)
+        )
+        style_name = (
+            style_name
+            or self.get_name(self.NAME_TYPOGRAPHIC_SUBFAMILY_NAME)
+            or self.get_name(self.NAME_SUBFAMILY_NAME)
+        )
+        full_name = f'{family_name} {style_name}'
+        postscript_family_name = family_name.replace(' ', '')
+        postscript_subfamily_name = style_name.replace(' ', '')
+        postscript_name = f'{postscript_family_name}-{postscript_subfamily_name}'
+        names = {
+            self.NAME_FAMILY_NAME: family_name,
+            self.NAME_SUBFAMILY_NAME: style_name,
+            self.NAME_FULL_NAME: full_name,
+            self.NAME_POSTSCRIPT_NAME: postscript_name,
+            self.NAME_TYPOGRAPHIC_FAMILY_NAME: family_name,
+            self.NAME_TYPOGRAPHIC_SUBFAMILY_NAME: style_name,
+        }
+        self.set_names(names=names)
+
     def save(self, filepath=None, overwrite=False):
         """
         Saves the font at filepath.
@@ -688,16 +745,11 @@ class Font(object):
         :type key: int or str
         :param value: The value
         :type value: str
-
-        :raises KeyError: if the key is not a valid name key/id
         """
-        name_id = self._get_name_id(key)
         font = self.get_ttfont()
-        for record in font['name'].names:
-            if record.nameID == name_id:
-                record.string = str(value or '')
-                return
-        raise KeyError(f'Invalid name key: {key}.')
+        name_id = self._get_name_id(key)
+        name_table = font['name']
+        name_table.setName(value, name_id, 3, 1, None)
 
     def set_names(self, names):
         """
@@ -709,49 +761,65 @@ class Font(object):
         for key, value in names.items():
             self.set_name(key, value)
 
-    @classmethod
-    def _set_bool_flag(self, bits, bit, flag):
-        mask = 1 << bit
-        bits = bits | mask if flag else bits & ~mask
-        return bits
+    def set_style_flag(self, key, value):
+        """
+        Sets the style flag.
 
-    def _set_style_flag(self, flag, value):
+        :param key: The flag key
+        :type key: str
+        :param value: The value
+        :type value: bool
+        """
         font = self.get_ttfont()
-        bits = self._STYLE_FLAGS.get(flag)
-        bit_os2_fs = bits.get('bit_os2_fs')
-        bit_head_mac = bits.get('bit_head_mac')
+        bits = self._STYLE_FLAGS[key]
+        bit_os2_fs = bits['bit_os2_fs']
+        bit_head_mac = bits['bit_head_mac']
         if bit_os2_fs is not None:
             os2 = font.get('OS/2')
             if os2:
-                os2.fsSelection = self._set_bool_flag(
-                    os2.fsSelection, bit_os2_fs, value
-                )
+                os2.fsSelection = set_flag(os2.fsSelection, bit_os2_fs, value)
         if bit_head_mac is not None:
             head = font.get('head')
             if head:
-                head.macStyle = self._set_bool_flag(head.macStyle, bit_head_mac, value)
+                head.macStyle = set_flag(head.macStyle, bit_head_mac, value)
 
     def set_style_flags(
-        self, regular=None, bold=None, italic=None, outline=None, underline=None
+        self,
+        regular=None,
+        bold=None,
+        italic=None,
+        underline=None,
+        outline=None,
+        shadow=None,
+        condensed=None,
+        extended=None,
     ):
         """
-        Sets the style flags, flags set to None will be ignored.
+        Sets the style flags, keys set to None will be ignored.
 
-        :param bold: The bold flag value.
+        :param regular: The regular style flag value
+        :type regular: bool or None
+        :param bold: The bold style flag value
         :type bold: bool or None
-        :param italic: The italic flag value.
+        :param italic: The italic style flag value
         :type italic: bool or None
-        :param underline: The underline flag value.
+        :param underline: The underline style flag value
         :type underline: bool or None
-        :param outline: The outline flag value.
+        :param outline: The outline style flag value
         :type outline: bool or None
+        :param shadow: The shadow style flag value
+        :type shadow: bool or None
+        :param condensed: The condensed style flag value
+        :type condensed: bool or None
+        :param extended: The extended style flag value
+        :type extended: bool or None
         """
         flags = locals()
         flags.pop('self')
-        for flag, value in flags.items():
-            assert flag in self._STYLE_FLAGS_KEYS
-            if isinstance(value, bool):
-                self._set_style_flag(flag, value)
+        for key, value in flags.items():
+            if value is not None:
+                assert isinstance(value, bool)
+                self.set_style_flag(key, value)
 
     def subset(self, unicodes='', glyphs=[], text='', **options):
         """
