@@ -2,8 +2,8 @@
 
 __author__ = 'Fabio Caccamo'
 __copyright__ = 'Copyright (c) 2021-present Fabio Caccamo'
-__description__ = 'human-friendly font operations.'
+__description__ = 'friendly font operations.'
 __email__ = 'fabio.caccamo@gmail.com'
 __license__ = 'MIT'
 __title__ = 'python-fontbro'
-__version__ = '0.1.0'
+__version__ = '0.2.0'
